from array import ArrayType, array
import os.path
from whatsapp_api_client_python.response import Response


class Groups:
    def __init__(self, greenApi) -> None:
        self.greenApi = greenApi
        
    def addGroupParticipant(self, 
            groupId: str, 
            participantChatId: str) -> Response:
            'The method adds a participant to a group chat.'

            requestBody = {
                'groupId': groupId,
                'participantChatId': participantChatId,
            }
            
            return self.greenApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/AddGroupParticipant/{{apiTokenInstance}}',
                requestBody)

    def createGroup(self, groupName: str, chatIds: array) -> Response:
            'The method is aimed for creating a group chat.'
            
            requestBody = {
                'groupName': groupName,
                'chatIds': chatIds
            }

            return self.greenApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/CreateGroup/{{apiTokenInstance}}',
                requestBody)

    def getGroupData(self, groupId: str) -> Response:
            'The method gets group chat data.'

            requestBody = {
                'groupId': groupId
            }
            
            return self.greenApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/GetGroupData/{{apiTokenInstance}}',
                requestBody)

    def leaveGroup(self, groupId: str) -> Response:
            'The method makes the current account user leave the group chat.'

            requestBody = {
                'groupId': groupId
            }
            
            return self.greenApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/LeaveGroup/{{apiTokenInstance}}',
                requestBody)

    def removeAdmin(self, groupId: str, participantChatId: str) -> Response:
            'The method removes a participant from group chat '\
            'administartion rights.'

            requestBody = {
                'groupId': groupId,
                'participantChatId': participantChatId
            }
            
            return self.greenApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/RemoveAdmin/{{apiTokenInstance}}',
                requestBody)

    def removeGroupParticipant(self, 
        groupId: str, 
        participantChatId: str) -> Response:
            'The method removes a participant from a group chat.'

            requestBody = {
                'groupId': groupId,
                'participantChatId': participantChatId
            }
            
            return self.greenApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/RemoveGroupParticipant/{{apiTokenInstance}}',
                requestBody)

    def setGroupAdmin(self, 
        groupId: str, 
        participantChatId: str) -> Response:
            'The method sets a group chat participant as an administrator.'

            requestBody = {
                'groupId': groupId,
                'participantChatId': participantChatId
            }
            
            return self.greenApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/SetGroupAdmin/{{apiTokenInstance}}',
                requestBody)

    def setGroupPicture(self, 
        groupId: str, 
        path: str) -> Response:
            'The method sets a group picture.'

            requestBody = {
                'groupId': groupId
            }

            pathParts = os.path.split(path)
            file = pathParts[1]

            files = [
                ('file',(file, open(path,'rb'),'image/jpeg'))
            ]
            
            return self.greenApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/SetGroupPicture/{{apiTokenInstance}}',
                requestBody, files)

    def updateGroupName(self, 
        groupId: str, 
        groupName: str) -> Response:
            'The method changes a group chat name.'

            requestBody = {
                'groupId': groupId,
                'groupName': groupName
            }
            
            return self.greenApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/UpdateGroupName/{{apiTokenInstance}}',
                requestBody)


from array import array
import requests
import json
from whatsapp_api_client_python.response import Response

from whatsapp_api_client_python.tools.account import Account
from whatsapp_api_client_python.tools.device import Device
from whatsapp_api_client_python.tools.groups import Groups
from whatsapp_api_client_python.tools.journals import Journals
from whatsapp_api_client_python.tools.marking import Marking
from whatsapp_api_client_python.tools.queues import Queues
from whatsapp_api_client_python.tools.receiving import Receiving
from whatsapp_api_client_python.tools.sending import Sending
from whatsapp_api_client_python.tools.serviceMethods import ServiceMethods
from whatsapp_api_client_python.tools.webhooks import Webhooks


class GreenApi:
    'REST API class'

    host: str
    idInstance: str
    apiTokenInstance: str

    def __init__(self, 
                    idInstance: str, 
                    apiTokenInstance: str,
                    host: str = 'https://api.green-api.com') -> None:
        self.host = host
        self.idInstance = idInstance
        self.apiTokenInstance = apiTokenInstance

        self.account = Account(self)
        self.device = Device(self)
        self.groups = Groups(self)
        self.journals = Journals(self)
        self.marking = Marking(self)
        self.queues = Queues(self)
        self.receiving = Receiving(self)
        self.sending = Sending(self)
        self.serviceMethods = ServiceMethods(self)
        self.webhooks = Webhooks(self)

    def request(self, method: str, url: str, 
                payload: any = None, files: array = None):
        url = url.replace('{{host}}', self.host)
        url = url.replace('{{idInstance}}', self.idInstance)
        url = url.replace('{{apiTokenInstance}}', self.apiTokenInstance)
        status_code = 0
        text = ''
        try:
            headers = {}
            payloadData = None
            if payload != None:
                if files == None:
                    headers = {
                        'Content-Type': 'application/json'
                    }
                    payloadData = json.dumps(payload)
                else:
                    payloadData = payload   
            result = requests.request(method, url, headers = headers, 
                                        data = payloadData, 
                                        files = files)
            status_code = result.status_code
            text = result.text
            result.raise_for_status()
        except requests.HTTPError:
            return Response(status_code, text)
        except Exception as err:
            status_code = 0
            text = f'Other error occurred: {err}'
        return Response(status_code, text)


from os import environ

from whatsapp_api_client_python import API as API


ID_INSTANCE = environ['ID_INSTANCE']
API_TOKEN_INSTANCE = environ['API_TOKEN_INSTANCE']

greenAPI = API.GreenApi(ID_INSTANCE, API_TOKEN_INSTANCE)

class TestClass: 
        def test_getSettings(self):            
                result = greenAPI.account.getSettings()
                if result.code != 200:
                        print(result.error)
                assert isinstance(result, API.Response) and result.code == 200

        def test_getStateInstance(self):            
                result = greenAPI.account.getStateInstance()
                if result.code != 200:
                        print(result.error)
                assert isinstance(result, API.Response) and result.code == 200

def main():
        TestClass.test_getSettings(TestClass)
        TestClass.test_getStateInstance(TestClass)

if __name__ == "__main__":
    main()


class Webhooks:
    def __init__(self, greenApi) -> None:
        self.greenApi = greenApi
        self.started = False

    def startReceivingNotifications(self, onEvent) -> bool:
        self.started = True
        self.job(onEvent)

    def stopReceivingNotifications(self) -> bool:
        self.started = False

    
    def job(self, onEvent) -> None:
        print('Incoming notifications are being received. '\
        'To interrupt, press Ctrl+C')
        try:
            while self.started == True:
                resultReceive = self.greenApi.receiving.receiveNotification()
                if resultReceive.code == 200:
                    if resultReceive.data == None:
                        # There are no incoming notifications, 
                        # we send the request again
                        continue
                    body = resultReceive.data['body']
                    typeWebhook = body['typeWebhook']
                    onEvent(typeWebhook, body)    
                    self.greenApi.receiving.deleteNotification(
                        resultReceive.data['receiptId'])
            print('End receiving')
        except KeyboardInterrupt:
            print('End receiving')
            pass


from whatsapp_api_client_python.response import Response

class Journals:
    def __init__(self, greenApi) -> None:
        self.greenApi = greenApi
        
    def getChatHistory(self, chatId: str, count: str) -> Response:
            'The method returns the chat message history.'

            requestBody = {
                'chatId': chatId,
                'count': count,
            }

            return self.greenApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/GetChatHistory/{{apiTokenInstance}}',
                requestBody)

    def lastIncomingMessages(self) -> Response:
            'The method returns the chat message history.'

            return self.greenApi.request('GET', 
                '{{host}}/waInstance{{idInstance}}'
                '/LastIncomingMessages/{{apiTokenInstance}}')

    def lastOutgoingMessages(self) -> Response:
            'The method returns the last outgoing messages of the account.'
            'Outgoing messages are stored on the server for 24 hours.'

            return self.greenApi.request('GET', 
                '{{host}}/waInstance{{idInstance}}'
                '/LastOutgoingMessages/{{apiTokenInstance}}')


import setuptools

with open("README.md") as file:
    read_me_description = file.read()

setuptools.setup(
    name="whatsapp-api-client-python",
    version="0.0.34",
    install_requires=['requests'],
    author="Ivan Sadovy",
    author_email="sadiv@bk.ru",
    description="This library helps you easily create a python '\
        'application to connect the WhatsApp API using service green-api.com",
    long_description=read_me_description,
    long_description_content_type="text/markdown",
    url="https://github.com/green-api/whatsapp-api-client-python",
    packages=['whatsapp_api_client_python', 'whatsapp_api_client_python.tools'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.5',
)